@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 import os
 import shutil
 from pkg_resources import resource_filename
+from PySide6 import QtCore, QtWidgets, QtGui
 
 from . import strings
 from onionshare_cli.onion import (
@@ -39,7 +40,8 @@ from onionshare_cli.onion import (
     TorTooOldStealth,
     PortNotAvailable,
 )
-
+from onionshare_cli.meek import Meek
+from onionshare_cli.web.web import WaitressException
 
 class GuiCommon:
     """
@@ -77,6 +79,9 @@ class GuiCommon:
             os.makedirs(self.events_dir, 0o700, True)
         self.events_filename = os.path.join(self.events_dir, "events")
 
+        # Instantiate Meek, which is used to bypass censorship
+        self.meek = Meek(self.common, get_tor_paths=self.get_tor_paths)
+
         self.css = self.get_css(qtapp.color_mode)
         self.color_mode = qtapp.color_mode
 
@@ -116,6 +121,15 @@ class GuiCommon:
                     font-weight: bold;
                     font-size: 20px;
                 }""",
+            "settings_subtab_bar": """
+                QTabBar::tab {
+                    background: transparent;
+                }
+                QTabBar::tab:selected {
+                    border-bottom: 3px solid;
+                    border-color: #4E064F;
+                    padding: 3px
+                }""",
             "mode_new_tab_button": """
                 QPushButton {
                     font-weight: bold;
@@ -149,6 +163,64 @@ class GuiCommon:
                 QStatusBar::item {
                     border: 0px;
                 }""",
+            "autoconnect_start_button": """
+                QPushButton {
+                    background-color: #5fa416;
+                    color: #ffffff;
+                    padding: 10px;
+                    border: 0;
+                    border-radius: 5px;
+                }""",
+            "autoconnect_configure_button": """
+                QPushButton {
+                    padding: 9px 29px;
+                    color: #3f7fcf;
+                    text-align: left;
+                }""",
+            "enable_autoconnect": """
+                QCheckBox {
+                    margin-top: 30px;
+                    background: #FCFCFC;
+                    color: #000000;
+                    border: 1px solid #DDDBDA;
+                    border-radius: 8px;
+                    padding: 24px 16px;
+                }
+                QCheckBox::indicator {
+                    width: 0;
+                    height: 0;
+                }""",
+            "autoconnect_countries_combobox": """
+                QComboBox {
+                    padding: 10px;
+                    font-size: 16px;
+                    margin-left: 32px;
+                }
+                QComboBox:disabled {
+                    color: #666666;
+                }
+                """,
+            "autoconnect_task_label": """
+                QLabel {
+                    font-weight: bold;
+                }
+                """,
+            "autoconnect_failed_to_connect_label": """
+                QLabel {
+                    font-size: 18px;
+                    font-weight: bold;
+                }""",
+            "autoconnect_bridge_setting_options": """
+                QGroupBox {
+                    border: 0;
+                    border-color: transparent;
+                    background-color: transparent;
+                    font-weight: bold;
+                    margin-top: 16px;
+                }
+                QGroupBox::title {
+                    subcontrol-origin: margin;
+                }""",
             # Common styles between modes and their child widgets
             "mode_settings_toggle_advanced": """
                 QPushButton {
@@ -399,6 +471,10 @@ class GuiCommon:
                 QPushButton {
                     padding: 5px 10px;
                 }""",
+            "receive_options": """
+                QCheckBox:disabled {
+                    color: #666666;
+                }""",
             # Tor Settings dialogs
             "tor_settings_error": """
                 QLabel {
@@ -435,12 +511,12 @@ class GuiCommon:
 
         if self.common.platform == "Windows":
             base_path = self.get_resource_path("tor")
-            tor_path = os.path.join(base_path, "Tor", "tor.exe")
-            obfs4proxy_file_path = os.path.join(base_path, "Tor", "obfs4proxy.exe")
-            snowflake_file_path = os.path.join(base_path, "Tor", "snowflake-client.exe")
-            meek_client_file_path = os.path.join(base_path, "Tor", "meek-client.exe")
-            tor_geo_ip_file_path = os.path.join(base_path, "Data", "Tor", "geoip")
-            tor_geo_ipv6_file_path = os.path.join(base_path, "Data", "Tor", "geoip6")
+            tor_path = os.path.join(base_path, "tor.exe")
+            obfs4proxy_file_path = os.path.join(base_path, "obfs4proxy.exe")
+            snowflake_file_path = os.path.join(base_path, "snowflake-client.exe")
+            meek_client_file_path = os.path.join(base_path, "meek-client.exe")
+            tor_geo_ip_file_path = os.path.join(base_path, "geoip")
+            tor_geo_ipv6_file_path = os.path.join(base_path, "geoip6")
         elif self.common.platform == "Darwin":
             base_path = self.get_resource_path("tor")
             tor_path = os.path.join(base_path, "tor")
@@ -508,3 +584,59 @@ class GuiCommon:
         elif type(e) is PortNotAvailable:
             return strings._("error_port_not_available")
         return None
+
+    @staticmethod
+    def get_translated_web_error(e):
+        """
+        Takes an exception defined in web.py and returns a translated error message
+        """
+        if type(e) is WaitressException:
+            return strings._("waitress_web_server_error")
+
+class ToggleCheckbox(QtWidgets.QCheckBox):
+    def __init__(self, text):
+        super(ToggleCheckbox, self).__init__(text)
+        # Set default parameters
+        self.setCursor(QtCore.Qt.PointingHandCursor)
+        self.w = 50
+        self.h = 24
+        self.bg_color = "#D4D4D4"
+        self.circle_color = "#BDBDBD"
+        self.active_color = "#4E0D4E"
+        self.inactive_color = ""
+
+    def hitButton(self, pos):
+        return self.toggleRect.contains(pos)
+
+    def paintEvent(self, e):
+        painter = QtGui.QPainter(self)
+        painter.setRenderHint(QtGui.QPainter.Antialiasing)
+        painter.setPen(QtCore.Qt.NoPen)
+        opt = QtWidgets.QStyleOptionButton()
+        opt.initFrom(self)
+        self.initStyleOption(opt)
+        s = self.style()
+        s.drawControl(QtWidgets.QStyle.CE_CheckBox, opt, painter, self)
+
+        rect = QtCore.QRect(
+            s.subElementRect(QtWidgets.QStyle.SE_CheckBoxContents, opt, self)
+        )
+        x = (
+            rect.width() - rect.x() - self.w + 20
+        )  # 20 is the padding between text and toggle
+        y = (
+            self.height() / 2 - self.h / 2 + 16
+        )  # 16 is the padding top for the checkbox
+        self.toggleRect = QtCore.QRect(x, y, self.w, self.h)
+        painter.setBrush(QtGui.QColor(self.bg_color))
+        painter.drawRoundedRect(x, y, self.w, self.h, self.h / 2, self.h / 2)
+        if not self.isChecked():
+            painter.setBrush(QtGui.QColor(self.circle_color))
+            painter.drawEllipse(x, y - 3, self.h + 6, self.h + 6)
+        else:
+            painter.setBrush(QtGui.QColor(self.active_color))
+            painter.drawEllipse(
+                x + self.w - (self.h + 6), y - 3, self.h + 6, self.h + 6
+            )
+
+        painter.end()
