@@ -25,7 +25,7 @@ import mimetypes
 import gzip
 from flask import Response, request
 from unidecode import unidecode
-from werkzeug.urls import url_quote
+from urllib.parse import quote
 
 
 class SendBaseModeWeb:
@@ -44,8 +44,9 @@ class SendBaseModeWeb:
         self.download_filesize = None
         self.zip_writer = None
 
-        # Store the tempfile objects here, so when they're garbage collected the files are deleted
-        self.gzip_files = []
+        # Create a temporary dir to store gzip files in
+        self.gzip_tmp_dir = tempfile.TemporaryDirectory(dir=self.common.build_tmp_dir())
+        self.gzip_counter = 0
 
         # If autostop_sharing, only allow one download at a time
         self.download_in_progress = False
@@ -130,7 +131,7 @@ class SendBaseModeWeb:
 
         self.set_file_info_custom(filenames, processed_size_callback)
 
-    def directory_listing(self, filenames, path="", filesystem_path=None):
+    def directory_listing(self, filenames, path="", filesystem_path=None, add_trailing_slash=False):
         # Tell the GUI about the directory listing
         history_id = self.cur_history_id
         self.cur_history_id += 1
@@ -149,12 +150,12 @@ class SendBaseModeWeb:
         breadcrumbs_leaf = breadcrumbs.pop()[0]
 
         # If filesystem_path is None, this is the root directory listing
-        files, dirs = self.build_directory_listing(path, filenames, filesystem_path)
+        files, dirs = self.build_directory_listing(path, filenames, filesystem_path, add_trailing_slash)
         return self.directory_listing_template(
             path, files, dirs, breadcrumbs, breadcrumbs_leaf
         )
 
-    def build_directory_listing(self, path, filenames, filesystem_path):
+    def build_directory_listing(self, path, filenames, filesystem_path, add_trailing_slash=False):
         files = []
         dirs = []
 
@@ -167,9 +168,14 @@ class SendBaseModeWeb:
             is_dir = os.path.isdir(this_filesystem_path)
 
             if is_dir:
-                dirs.append(
-                    {"link": os.path.join(f"/{path}", filename), "basename": filename}
-                )
+                if add_trailing_slash:
+                    dirs.append(
+                        {"link": os.path.join(f"/{path}", filename, ""), "basename": filename}
+                    )
+                else:
+                    dirs.append(
+                        {"link": os.path.join(f"/{path}", filename), "basename": filename}
+                    )
             else:
                 size = os.path.getsize(this_filesystem_path)
                 size_human = self.common.human_readable_filesize(size)
@@ -193,15 +199,12 @@ class SendBaseModeWeb:
         # gzip compress the individual file, if it hasn't already been compressed
         if use_gzip:
             if filesystem_path not in self.gzip_individual_files:
-                self.gzip_files.append(
-                    tempfile.NamedTemporaryFile("wb+", dir=self.common.build_tmp_dir())
+                gzip_filename = os.path.join(
+                    self.gzip_tmp_dir.name, str(self.gzip_counter)
                 )
-                gzip_file = self.gzip_files[-1]
-                self._gzip_compress(filesystem_path, gzip_file.name, 6, None)
-                self.gzip_individual_files[filesystem_path] = gzip_file.name
-
-                # Cleanup this temp file
-                self.web.cleanup_tempfiles.append(gzip_file)
+                self.gzip_counter += 1
+                self._gzip_compress(filesystem_path, gzip_filename, 6, None)
+                self.gzip_individual_files[filesystem_path] = gzip_filename
 
             file_to_download = self.gzip_individual_files[filesystem_path]
             filesize = os.path.getsize(self.gzip_individual_files[filesystem_path])
@@ -286,7 +289,7 @@ class SendBaseModeWeb:
         r.headers.set("Content-Length", filesize)
         filename_dict = {
             "filename": unidecode(basename),
-            "filename*": "UTF-8''%s" % url_quote(basename),
+            "filename*": "UTF-8''%s" % quote(basename),
         }
         r.headers.set("Content-Disposition", "inline", **filename_dict)
         (content_type, _) = mimetypes.guess_type(basename, strict=False)
