@@ -2,7 +2,7 @@
 """
 OnionShare | https://onionshare.org/
 
-Copyright (C) 2014-2021 Micah Lee, et al. <micah@micahflee.com>
+Copyright (C) 2014-2022 Micah Lee, et al. <micah@micahflee.com>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ from datetime import datetime, timezone
 from flask import Response, request, render_template, make_response, abort
 from unidecode import unidecode
 from werkzeug.http import parse_date, http_date
-from werkzeug.urls import url_quote
+from urllib.parse import quote
 
 from .send_base_mode import SendBaseModeWeb
 
@@ -134,8 +134,12 @@ class ShareModeWeb(SendBaseModeWeb):
         The web app routes for sharing files
         """
 
-        @self.web.app.route("/", defaults={"path": ""}, methods=["GET"], provide_automatic_options=False)
-        @self.web.app.route("/<path:path>", methods=["GET"], provide_automatic_options=False)
+        @self.web.app.route(
+            "/", defaults={"path": ""}, methods=["GET"], provide_automatic_options=False
+        )
+        @self.web.app.route(
+            "/<path:path>", methods=["GET"], provide_automatic_options=False
+        )
         def index(path):
             """
             Render the template for the onionshare landing page.
@@ -159,7 +163,9 @@ class ShareModeWeb(SendBaseModeWeb):
 
             return self.render_logic(path)
 
-        @self.web.app.route("/download", methods=["GET"], provide_automatic_options=False)
+        @self.web.app.route(
+            "/download", methods=["GET"], provide_automatic_options=False
+        )
         def download():
             """
             Download the zip file.
@@ -175,7 +181,6 @@ class ShareModeWeb(SendBaseModeWeb):
 
             # Prepare some variables to use inside generate() function below
             # which is outside of the request context
-            shutdown_func = request.environ.get("werkzeug.server.shutdown")
             request_path = request.path
 
             # If this is a zipped file, then serve as-is. If it's not zipped, then,
@@ -212,7 +217,6 @@ class ShareModeWeb(SendBaseModeWeb):
             else:
                 r = Response(
                     self.generate(
-                        shutdown_func,
                         range_,
                         file_to_download,
                         request_path,
@@ -227,7 +231,7 @@ class ShareModeWeb(SendBaseModeWeb):
             r.headers.set("Content-Length", range_[1] - range_[0] + 1)
             filename_dict = {
                 "filename": unidecode(basename),
-                "filename*": "UTF-8''%s" % url_quote(basename),
+                "filename*": "UTF-8''%s" % quote(basename),
             }
             r.headers.set("Content-Disposition", "attachment", **filename_dict)
             # guess content type
@@ -286,7 +290,9 @@ class ShareModeWeb(SendBaseModeWeb):
         if if_unmod:
             if_date = parse_date(if_unmod)
             if if_date and not if_date.tzinfo:
-                if_date = if_date.replace(tzinfo=timezone.utc) # Compatible with Flask < 2.0.0
+                if_date = if_date.replace(
+                    tzinfo=timezone.utc
+                )  # Compatible with Flask < 2.0.0
             if if_date and if_date > last_modified:
                 abort(412)
             elif range_header is None:
@@ -295,7 +301,7 @@ class ShareModeWeb(SendBaseModeWeb):
         return range_, status_code
 
     def generate(
-        self, shutdown_func, range_, file_to_download, path, history_id, filesize
+        self, range_, file_to_download, path, history_id, filesize
     ):
         # The user hasn't canceled the download
         self.client_cancel = False
@@ -382,9 +388,7 @@ class ShareModeWeb(SendBaseModeWeb):
             print("Stopped because transfer is complete")
             self.web.running = False
             try:
-                if shutdown_func is None:
-                    raise RuntimeError("Not running with the Werkzeug Server")
-                shutdown_func()
+                self.web.stop()
             except Exception:
                 pass
 
@@ -459,7 +463,7 @@ class ShareModeWeb(SendBaseModeWeb):
                 return self.web.error404(history_id)
 
     def build_zipfile_list(self, filenames, processed_size_callback=None):
-        self.common.log("ShareModeWeb", "build_zipfile_list")
+        self.common.log("ShareModeWeb", "build_zipfile_list", f"filenames={filenames}")
         for filename in filenames:
             info = {
                 "filename": filename,
@@ -484,7 +488,10 @@ class ShareModeWeb(SendBaseModeWeb):
                 self.download_etag = make_etag(f)
 
             # Compress the file with gzip now, so we don't have to do it on each request
-            self.gzip_filename = tempfile.mkstemp("wb+")[1]
+            self.gzip_tmp_dir = tempfile.TemporaryDirectory(
+                dir=self.common.build_tmp_dir()
+            )
+            self.gzip_filename = os.path.join(self.gzip_tmp_dir.name, "file.gz")
             self._gzip_compress(
                 self.download_filename, self.gzip_filename, 6, processed_size_callback
             )
@@ -492,15 +499,15 @@ class ShareModeWeb(SendBaseModeWeb):
             with open(self.gzip_filename, "rb") as f:
                 self.gzip_etag = make_etag(f)
 
-            # Make sure the gzip file gets cleaned up when onionshare stops
-            self.web.cleanup_filenames.append(self.gzip_filename)
-
             self.is_zipped = False
 
+            # Cleanup this tempfile
+            self.web.cleanup_tempdirs.append(self.gzip_tmp_dir)
+
         else:
             # Zip up the files and folders
             self.zip_writer = ZipWriter(
-                self.common, processed_size_callback=processed_size_callback
+                self.common, self.web, processed_size_callback=processed_size_callback
             )
             self.download_filename = self.zip_writer.zip_filename
             for info in self.file_info["files"]:
@@ -519,10 +526,6 @@ class ShareModeWeb(SendBaseModeWeb):
             with open(self.download_filename, "rb") as f:
                 self.download_etag = make_etag(f)
 
-            # Make sure the zip file gets cleaned up when onionshare stops
-            self.web.cleanup_filenames.append(self.zip_writer.zip_filename)
-            self.web.cleanup_filenames.append(self.zip_writer.zip_temp_dir)
-
             self.is_zipped = True
 
         return True
@@ -535,17 +538,24 @@ class ZipWriter(object):
     filename.
     """
 
-    def __init__(self, common, zip_filename=None, processed_size_callback=None):
+    def __init__(
+        self, common, web=None, zip_filename=None, processed_size_callback=None
+    ):
         self.common = common
+        self.web = web
         self.cancel_compression = False
 
         if zip_filename:
             self.zip_filename = zip_filename
         else:
-            self.zip_temp_dir = tempfile.mkdtemp()
-            self.zip_filename = (
-                f"{self.zip_temp_dir}/onionshare_{self.common.random_string(4, 6)}.zip"
+            self.zip_temp_dir = tempfile.TemporaryDirectory(
+                dir=self.common.build_tmp_dir()
             )
+            self.zip_filename = f"{self.zip_temp_dir.name}/onionshare_{self.common.random_string(4, 6)}.zip"
+
+            # Cleanup this temp dir
+            if self.web:
+                self.web.cleanup_tempdirs.append(self.zip_temp_dir)
 
         self.z = zipfile.ZipFile(self.zip_filename, "w", allowZip64=True)
         self.processed_size_callback = processed_size_callback
